import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { DictDataService } from '../../service/system/dictData.service';
import { CDTO_dictData, CVO_dictData, FVO_dictData, QDTO_dictData, QVO_dictData, UDTO_dictData, UVAO_dictData } from '../../interface/system/dictData.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['字典数据表'])
@Controller('/dictData')
export class DictDataController extends BaseController {
  @Inject()
  service: DictDataService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CVO_dictData })
  async create(@Body() data: CDTO_dictData) {
    const dbData = await this.service.create(data);
    const result = new CVO_dictData(dbData);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_dictData })
  async query(@Query() filter: QDTO_dictData, @Query('skip') skip: number, @Query('limit') limit: number) {
    const list = await this.service.query(filter, { skip, limit });
    const data = [];
    for (const i of list) {
      const newData = new QVO_dictData(i);
      data.push(newData);
    }
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FVO_dictData })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FVO_dictData(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UVAO_dictData })
  async update(@Param('id') id: string, @Body() body: UDTO_dictData) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
