import {
  JoinPoint,
  MidwayDecoratorService,
  REQUEST_OBJ_CTX_KEY,
  createCustomMethodDecorator,
} from '@midwayjs/core';
import { FrameworkErrorEnum, ServiceError } from 'free-midway-component';
export const VERIFYTOKEN_KEY = 'decorator:verify_token';
/**
 *
 * 只检测是否有token,此装饰器不做具体用token处理什么.
 */
export function verifyToken() {
  return createCustomMethodDecorator(VERIFYTOKEN_KEY, {});
}
/**
 * 验证token装饰器实现
 * @param decoratorService 装饰器服务
 */
export function VerifyTokenInit(decoratorService: MidwayDecoratorService) {
  decoratorService.registerMethodHandler(VERIFYTOKEN_KEY, options => {
    return {
      around: async (joinPoint: JoinPoint) => {
        const instance = joinPoint.target;
        const ctx = instance[REQUEST_OBJ_CTX_KEY];
        const user = ctx.user;
        if (!user)
          throw new ServiceError(
            '未检测到登录信息,无法访问接口!',
            FrameworkErrorEnum.NOT_LOGIN
          );
        const result = await joinPoint.proceed(...joinPoint.args);
        return result;
      },
    };
  });
}
